"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const path_1 = require("path");
const fs_1 = require("fs");
const glob_1 = __importDefault(require("glob"));
const resolver_js_1 = require("vite/dist/node/resolver.js");
const modulesDir = path_1.join(process.cwd(), '/node_modules/');
function template(template) {
    return (data) => {
        return template.replace(/#([^#]+)#/g, (_, g1) => data[g1] || g1);
    };
}
const globbyTransfrom = function (config) {
    const resolver = resolver_js_1.createResolver(config.root || process.cwd(), config.resolvers || [], config.alias || {});
    const cache = new Map();
    return {
        test({ path }) {
            const filePath = path.replace('\u0000', ''); // why some path startsWith '\u0000'?
            try {
                return (!filePath.startsWith(modulesDir) &&
                    /\.(vue|js|jsx|ts|tsx)$/.test(filePath) &&
                    fs_1.lstatSync(filePath).isFile());
            }
            catch {
                return false;
            }
        },
        transform({ code, path, isBuild }) {
            let result = cache.get(path);
            if (!result) {
                result = code.replace(/import\s+([\w\s{}\*]+)\s+from\s+(['"])globby!([^'"]+)\2/g, (_, g1, g2, g3) => {
                    const filePath = path.replace('\u0000', ''); // why some path startsWith '\u0000'?
                    // resolve path
                    const resolvedFilePath = g3.startsWith('.')
                        ? resolver.resolveRelativeRequest(filePath, g3)
                        : { pathname: resolver.requestToFile(g3) };
                    const files = glob_1.default.sync(resolvedFilePath.pathname, { dot: true });
                    let templateStr = 'import #name# from #file#'; // import default
                    let name = g1;
                    const m = g1.match(/\{\s*(\w+)(\s+as\s+(\w+))?\s*\}/); // import module
                    const m2 = g1.match(/\*\s+as\s+(\w+)/); // import * as all module
                    if (m) {
                        templateStr = `import { ${m[1]} as #name# } from #file#`;
                        name = m[3] || m[1];
                    }
                    else if (m2) {
                        templateStr = 'import * as #name# from #file#';
                        name = m2[1];
                    }
                    const temRender = template(templateStr);
                    const groups = [];
                    const replaceFiles = files.map((f, i) => {
                        const file = g2 + resolver.fileToRequest(f) + g2;
                        groups.push([name + i, file]);
                        return temRender({ name: name + i, file });
                    });
                    return (replaceFiles.join('\n') +
                        // '\n' +
                        // groups.map((v) => `${v[0]}._path = ${v[1]}`).join('\n') +
                        `\nconst ${name} = { ${groups.map((v) => v[0]).join(',')} }\n`);
                });
                if (isBuild)
                    cache.set('path', result);
            }
            return result;
            // console.log(ctx.code)
            // return ctx
        }
    };
};
module.exports = globbyTransfrom;
//# sourceMappingURL=index.js.map